"""
License: BSD

 van.frag (van.frag@gmail.com)
"""
import re
import md5

from trac import __version__
from trac.core import *
from trac.util.text import CRLF
from trac.wiki.model import WikiPage
from trac.versioncontrol.diff import unified_diff
from trac.notification import NotifyEmail
from trac.config import Option, BoolOption, ListOption, IntOption
from tracfullblog.model import BlogPost, BlogComment

from genshi.template.text import TextTemplate


class BlogNotifierEmail(NotifyEmail):
    template_name = "blog_notifier_email_template.txt"
    from_email = 'trac+wiki@localhost'
    blocked_addresses = []

    def __init__(self, env):
        NotifyEmail.__init__(self, env)

        self.from_name = self.config.get('blog-notifier', 'from_name')
        self.banned_addresses = []

    def notify(self, bp, action, 
               version=None, 
               time=None,
               comment=None, 
               author=None):
        self.bp = bp
        self.change_author = author
        self.time = time
        self.action = action
        self.version = version

        self.data['name']= bp.name
        self.data['title']= bp.title
        self.data['body']= bp.body
        self.data['comment']= comment
        self.data['version']= version
        self.data['author']= author
        self.data['action']= action
        self.data['time'] = time
        self.data['link']= self.env.abs_href.blog(bp.name)
        
        subject = self.format_subject(action.replace('_', ' '))

        NotifyEmail.notify(self, bp.name, subject)

    def get_recipients(self, pagename):
        
        tos = self.config.getlist("blog-notifier","smtp_always_bcc")
        
        self.env.log.debug("TO's TO NOTIFY: %s", tos)
        return (tos, [])

    def send(self, torcpts, ccrcpts, mime_headers={}):
        from email.MIMEText import MIMEText
        from email.Utils import formatdate

        stream = self.template.generate(**self.data)
        body = stream.render('text')
        projname = self.config.get('project', 'name')
        headers = {}
        headers['X-Mailer'] = 'Trac %s, by Edgewall Software' % __version__
        headers['X-Trac-Version'] =  __version__
        headers['X-Trac-Project'] =  projname
        headers['X-URL'] = self.config.get('project', 'url')
        headers['Precedence'] = 'bulk'
        headers['Auto-Submitted'] = 'auto-generated'
        headers['Subject'] = self.subject
        headers['From'] = (self.from_name or projname, self.from_email)
        headers['Reply-To'] = self.replyto_email

        def build_addresses(rcpts):
            """Format and remove invalid addresses"""
            return filter(lambda x: x, \
                          [self.get_smtp_address(addr) for addr in rcpts])

        def remove_dup(rcpts, all):
            """Remove duplicates"""
            tmp = []
            for rcpt in rcpts:
                if rcpt in self.banned_addresses:
                    self.env.log.debug("Banned Address: %s", rcpt)
                    self.blocked_addresses.append(rcpt)
                elif not rcpt in all:
                    tmp.append(rcpt)
                    all.append(rcpt)
            return (tmp, all)

        toaddrs = build_addresses(torcpts)
        ccaddrs = build_addresses(ccrcpts)
        accaddrs = []
        bccparam = self.config.getlist('blog-notifier', 'smtp_always_bcc')
        bccaddrs = bccparam and build_addresses(bccparam) or []

        recipients = []
        (toaddrs, recipients) = remove_dup(toaddrs, recipients)
        (ccaddrs, recipients) = remove_dup(ccaddrs, recipients)
        (accaddrs, recipients) = remove_dup(accaddrs, recipients)
        (bccaddrs, recipients) = remove_dup(bccaddrs, recipients)

        self.env.log.debug("Not notifying the following addresses: %s",
                           ', '.join(self.blocked_addresses))

        # if there is not valid recipient, leave immediately
        if len(recipients) < 1:
            self.env.log.info('no recipient for a wiki notification')
            return

        dest = self.change_author or 'anonymous'
        headers['X-Trac-Wiki-URL'] = self.data['link']

        pcc = accaddrs
        if pcc:
            headers['Cc'] = ', '.join(pcc)
        headers['Date'] = formatdate()
        # sanity check
        if not self._charset.body_encoding:
            try:
                dummy = body.encode('ascii')
            except UnicodeDecodeError:
                raise TracError(_("WikiPage contains non-Ascii chars. " \
                                  "Please change encoding setting"))

        msg = MIMEText(body, 'plain')
        # Message class computes the wrong type from MIMEText constructor,
        # which does not take a Charset object as initializer. Reset the
        # encoding type to force a new, valid evaluation
        del msg['Content-Transfer-Encoding']
        msg.set_charset(self._charset)

        self.add_headers(msg, headers);
        self.add_headers(msg, mime_headers);
        self.env.log.info("Sending SMTP notification to %s:%d to %s"
                           % (self.smtp_server, self.smtp_port, recipients))
        msgtext = msg.as_string()
        # Ensure the message complies with RFC2822: use CRLF line endings
        recrlf = re.compile("\r?\n")
        msgtext = CRLF.join(recrlf.split(msgtext))
        try:
            self.server.sendmail(msg['From'], recipients, msgtext)
        except Exception, err:
            self.env.log.debug('Notification could not be sent: %r', err)

    def format_subject(self, action):

        prefix = '[%s]' % self.config.get('project', 'name')

        prefix = prefix + ' Blog ' + self.bp.title
       
        return prefix.strip()